﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using dotMath2;

namespace dotMath2Test
{
    /// <summary>
    /// Summary description for TokenTests
    /// </summary>
    [TestClass]
    public class TokenTests
    {
        public TokenTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void Token_ctor_EmptyInit()
        {
            Token token = new Token();

            Assert.AreEqual<string>(string.Empty, token.ToString());
            Assert.AreEqual<CharType>(CharType.CT_UNDEF, token.TokenType);
        }

        [TestMethod]
        public void Token_ctor_StringInit()
        {
            string tokenValue = "test";
            CharType type = CharType.CT_LETTER;

            Token token = new Token(tokenValue, type);

            Assert.AreEqual<string>(tokenValue, token.ToString());
            Assert.AreEqual<CharType>(type, token.TokenType);
        }

        [TestMethod]
        public void Token_ctor_CharInit()
        {
            char tokenValue = 't';
            CharType type = CharType.CT_LETTER;

            Token token = new Token(tokenValue, type);

            Assert.AreEqual<string>("t", token.ToString());
            Assert.AreEqual<CharType>(type, CharType.CT_LETTER);
        }

        [TestMethod]
        public void Token_GetHashCode()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.AreEqual<int>(tokenValue.GetHashCode(), token.GetHashCode());
        }

        [TestMethod]
        public void Token_GetHashCode_Empty()
        {
            string tokenValue = string.Empty;
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.AreEqual<int>(tokenValue.GetHashCode(), token.GetHashCode());
        }

        [TestMethod]
        public void Token_GetHashCode_Null()
        {
            string tokenValue = null;
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.AreEqual<int>(string.Empty.GetHashCode(), token.GetHashCode());
        }

        [TestMethod]
        public void Token_Equals_string()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsTrue(token.Equals(tokenValue));
            Assert.IsFalse(token.Equals(tokenValue + "break"));
            Assert.IsFalse(token.Equals(null));

        }

        [TestMethod]
        public void Token_Equals_Token()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);
            Token tokenTest = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsTrue(token.Equals(tokenTest));
        }

        [TestMethod]
        public void Token_OpEquals_true()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsTrue(token == tokenValue);

        }

        [TestMethod]
        public void Token_OpEquals_false()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsFalse(token == tokenValue + "fail");
        }

        [TestMethod]
        public void Token_OpEquals_null()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsFalse(token == null);
        }

        [TestMethod]
        public void Token_OpNotEquals_true()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsTrue(token != tokenValue + "break");
        }

        [TestMethod]
        public void Token_OpNotEquals_false()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsFalse(token != tokenValue);
        }

        [TestMethod]
        public void Token_OpNotEquals_null()
        {
            string tokenValue = "test";
            Token token = new Token(tokenValue, CharType.CT_LETTER);

            Assert.IsFalse(token != null);
        }

    }
}
